# -*- coding: utf-8 -*-
import os
import dictdiffer
# yaml

import yaml
from yaml import Loader

# django Imports
from django.db.models.signals import post_delete, post_save,pre_save
from django.dispatch import receiver
from django.conf import settings

# App Imports
from track_actions.constants import TABLES
from track_actions.models import History
from track_actions.requestMiddleware import RequestMiddleware

# python imports
import datetime
import json

# def show_change(olddict,newdict):
#     """比较两个字典 返回如  [{'field': 'data.sex', 'old': '\xe7\x94\xb7', 'new': '\xe5\xa5\xb3'}, {'field': 'template', 'old': '', 'new': '11'}] """
#     changelist = []
#
#     for diff in list(dictdiffer.diff(olddict, newdict)):
#         changedict = {}
#         diff  = list(diff)
#         if diff[0] == "change":
#             changedict["field"] = diff[1]
#             changedict["old"] = diff[2][0]
#             changedict["new"] = diff[2][1]
#             changelist.append(changedict)
#     return changelist
#
#
#
#
# @receiver(pre_save)
# @receiver(post_delete)
# def track_user_actions(sender, instance, **kwargs):
#     """Signal function to track every change to a model
#
#     Arguments:
#         sender {object} -- The model sending the signal
#         instance {object} -- data instance
#     """
#
#     # print("updayefields",kwargs)
#     # 拿到实例后跟实例之前的进行比较 保存到一个记录里面 不要保存到多条记录里面
#
#     # 先获取原有的序列化的内容   监听应该在presave
#
#
#
#     current_request = RequestMiddleware.get_request_data()[1]
#
#     # 获取序列化器
#     request_url =current_request.path
#
#     if (
#         sender._meta.db_table not in TABLES
#         and hasattr(current_request, "user")
#         and hasattr(instance, "id")
#     ):
#         if RequestMiddleware.get_request_data()[0]:
#             request_data = decode_json(RequestMiddleware.get_request_data()[0])
#         else:
#             request_data = ""
#         data = instance.__dict__.copy()
#         data.__delitem__("_state")
#         if "drf_history.yaml" in os.listdir(settings.BASE_DIR):
#             yaml_file = open(settings.BASE_DIR + "/drf_history.yaml", "r")
#             yaml_content = yaml.load(yaml_file, Loader=Loader)
#             if (
#                 type(yaml_content) is dict
#                 and type(request_data) is dict
#                 and "fields_to_exclude" in yaml_content.keys()
#                 and type(yaml_content["fields_to_exclude"]) is list
#             ):
#                 feilds_to_exclude = yaml_content["fields_to_exclude"]
#                 [request_data.pop(key, None) for key in feilds_to_exclude]
#
#         save_history(instance, current_request, request_data, data)
#
#
# def decode_json(data):
#     request_data = ""
#     try:
#         request_data = json.loads(data)
#     except:
#         pass
#     return request_data
#
#
# def save_history(instance, current_request, request_data, data):
#     try:
#         history = History(
#             table_name=str(instance._meta.db_table),
#             user=current_request.user,
#             instance_id=instance.id,
#             action=current_request.method,
#             request_data=request_data,
#             path=current_request.path,
#             response_data=data,
#         )
#         history.save()
#     except ValueError:
#         pass
#
#
# if __name__ == '__main__':
#     second_dict = {
#         "template": "",
#         "template2": "",
#         "data": {
#             "name": "鸣人",
#             "age": 22,
#             "sex": "男",
#             "title": "六代火影"
#         }  # 数据
#     }
#
#     first_dict = {
#         "template": "11",
#         "template1": "11",
#         "data": {
#             "name": "鸣人",
#             "age": 22,
#             "sex": "女",
#             "title": "六代火影"
#         }  # 数据
#     }
#     change = show_change(second_dict,first_dict)
#     print(change)


from rest_framework import serializers
from track_actions.models import History


class HistorySerializer(serializers.ModelSerializer):
    class Meta:
        model = History
        fields = (
            "id",
            "table_name",
            "instance_id",
            "action",
            "user",
            "created_at",
        )



# Generated by Django 2.1.7 on 2019-10-15 08:59

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [migrations.swappable_dependency(settings.AUTH_USER_MODEL)]

    operations = [
        migrations.CreateModel(
            name="History",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("table_name", models.CharField(editable=False, max_length=255)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("instance_id", models.CharField(editable=False, max_length=255)),
                (
                    "action",
                    models.CharField(
                        choices=[
                            ("POST", "POST"),
                            ("DELETE", "DELETE"),
                            ("PUT", "PUT"),
                            ("PATCH", "PATCH"),
                        ],
                        editable=False,
                        max_length=7,
                    ),
                ),
                ("request_data", models.TextField(default="", editable=False)),
                ("response_data", models.TextField(default="", editable=False)),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={"verbose_name_plural": "History model"},
        )
    ]



# Generated by Django 2.2.3 on 2019-12-23 13:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('track_actions', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='history',
            name='path',
            field=models.CharField(default='', editable=False, max_length=255),
        ),
    ]



# Third-Party Imports
# from django.urls import path

from django.conf.urls import url, include
from rest_framework.routers import SimpleRouter
from track_actions.views import HistoryView

router = SimpleRouter()
router.register("history", HistoryView, "history")

urlpatterns = [
    url(r'^', include(router.urls, namespace='history')),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2021-06-08 16:07
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('track_actions', '0002_history_path'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='history',
            name=b'path',
        ),
        migrations.RemoveField(
            model_name='history',
            name=b'request_data',
        ),
        migrations.RemoveField(
            model_name='history',
            name=b'response_data',
        ),
        migrations.AlterField(
            model_name='history',
            name='action',
            field=models.TextField(blank=True, null=True),
        ),
    ]



from rest_framework.viewsets import ModelViewSet
from rest_framework.permissions import IsAuthenticated, IsAdminUser
from track_actions.serializers import HistorySerializer
from track_actions.models import History


class HistoryView(ModelViewSet):
    permission_classes = (IsAuthenticated, IsAdminUser)
    serializer_class = HistorySerializer
    queryset = History.objects.all()
    http_method_names = ["get"]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2021-06-09 17:26
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('track_actions', '0004_history_field_name'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='history',
            options={'ordering': ('-created_at',), 'verbose_name_plural': 'History model'},
        ),
    ]



# -*- coding: utf-8 -*-
import os
from setuptools import find_packages, setup

# with open(os.path.join(os.path.dirname(__file__), "README.md"), encoding='utf-8') as readme:
#     README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name="drf-api-history",
    version="1.1.3",
    packages=[
        "track_actions",
        "track_actions.migrations",
    ],
    description="drf api 访问历史记录 从api角度出发记录操作历史.",
    long_description='请到 https://pypi.org/project/drf-safe-jack/  查看完整的安装说明 这是drf-safe-jack的一个子库  ',
    long_description_content_type='text/markdown',
    keywords="djangorestframework drf history django audit tracking",
    author="blackjack0v0",
    author_email="yihongwy1@163.com",
    install_requires=[
        'pyyaml',
    ],
    url="",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 2.1",
        "Framework :: Django :: 2.2",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: BSD License",
    ],
)
